package eu.mhsl.craftattack.spawn.appliances.settings.datatypes;

import eu.mhsl.craftattack.spawn.Main;
import eu.mhsl.craftattack.spawn.appliances.settings.Settings;
import org.bukkit.Material;
import org.bukkit.NamespacedKey;
import org.bukkit.entity.Player;
import org.bukkit.inventory.ItemStack;
import org.bukkit.inventory.meta.ItemMeta;
import org.bukkit.persistence.PersistentDataContainer;

public abstract class Setting<TDataType> {
    private final Settings.Key key;

    public Setting(Settings.Key key) {
        this.key = key;
    }

    public NamespacedKey getNamespacedKey() {
        return new NamespacedKey(Main.instance(), key.name());
    }

    public Settings.Key getKey() {
        return key;
    }

    public void initializeFromPlayer(Player p) {
        fromStorage(p.getPersistentDataContainer());
    }

    public void triggerChange(Player p) {
        this.change();
        toStorage(p.getPersistentDataContainer(), this.state());
    }

    public ItemStack buildItem() {
        ItemStack stack = new ItemStack(icon(), 1);
        stack.setItemMeta(buildMeta(stack.getItemMeta()));
        return stack;
    }

    protected abstract Material icon();
    public abstract ItemMeta buildMeta(ItemMeta meta);
    protected abstract void change();
    protected abstract void fromStorage(PersistentDataContainer container);
    protected abstract void toStorage(PersistentDataContainer container, TDataType value);
    public abstract Class<?> dataType();
    public abstract TDataType state();
}
